# MXNet FAQ

This section addresses common questions about how to use _MXNet_. These include performance issues, e.g., how to train with multiple GPUs.
They also include workflow questions, e.g., how to visualize a neural network computation graph.
These answers are fairly focused. For more didactic, self-contained introductions to neural networks
and full working examples, visit the [tutorials section](../tutorials/index.md).


## Modeling
* [How do I fine-tune pre-trained models to a new dataset?](http://mxnet.io/how_to/finetune.html)

* [How do I work with variable-length input in MXNet (bucketing)?](http://mxnet.io/how_to/bucketing.html)

* [How do I visualize neural networks as computation graphs?](http://mxnet.io/how_to/visualize_graph.html)


## Speed

* [How can I train with multiple CPU/GPUs with data parallelism?](http://mxnet.io/how_to/multi_devices.html)

* [How can I train with multiple GPUs with model parallelism?](http://mxnet.io/how_to/model_parallel_lstm.html)

* [Can I use nnpack to improve the CPU performance of MXNet?](http://mxnet.io/how_to/nnpack.html)

* [What are the best setup and data-handling tips and tricks for improving speed?](http://mxnet.io/how_to/perf.html)


## Deployment Environments
* [Can I run MXNet on smart or mobile devices?](http://mxnet.io/how_to/smart_device.html)

* [How to use data from S3 for training?](s3_integration.md)

* [How to setup MXNet on AWS?](http://docs.aws.amazon.com/mxnet/latest/dg/mxnet-on-ec2-instance.html)

* [How to do distributed training using MXNet on AWS?](http://docs.aws.amazon.com/mxnet/latest/dg/mxnet-on-ec2-cluster.html)

* [How do I run MXNet on a Raspberry Pi for computer vision?](http://mxnet.io/tutorials/embedded/wine_detector.html)

* [How do I run Keras 1.2.2 with mxnet backend?](https://github.com/dmlc/keras/wiki/Installation)

* [How to convert MXNet models to Apple CoreML format?](https://github.com/apache/incubator-mxnet/tree/master/tools/coreml)

## Extend and Contribute to MXNet

* [How do I join the MXNet development discussion?](http://mxnet.io/community/mxnet_channels.html)

* [How do I contribute a patch to MXNet?](http://mxnet.io/community/contribute.html)

* [How do I implement operators in MXNet backend?](http://mxnet.io/how_to/add_op_in_backend.html)

* [How do I create new operators in MXNet?](http://mxnet.io/how_to/new_op.html)

* [How do I set MXNet's environmental variables?](http://mxnet.io/how_to/env_var.html)

* [How do I use MXNet as a front end for Torch?](http://mxnet.io/how_to/torch.html)

## Questions about Using MXNet
If you need help with using MXNet, have questions about applying it to a particular kind of problem, or have a discussion topic, please use our [forum](https://discuss.mxnet.io).

## Issue Tracker
We track bugs and new feature requests in the MXNet Github repo in the issues folder: [mxnet/issues](https://github.com/dmlc/mxnet/issues).

## Roadmap
MXNet is evolving fast. To see what's next and what we are working on internally, go to the [MXNet Roadmap](https://github.com/dmlc/mxnet/labels/Roadmap).
